@@ -1131,16 +1131,15 @@ def _process_repo_serial(git_repo_dir, sqlite_db_file, commits, extraction_setti
 
     git_repo = pydriller.GitRepository(git_repo_dir)
 
-    con = sqlite3.connect(sqlite_db_file)
-
     for commit in tqdm(commits, desc='Serial'):
         args = {'git_repo_dir': git_repo_dir, 'commit_hash': commit.hash, 'extraction_settings': extraction_settings}
         result = _process_commit(args)
 
-        if not result['edits'].empty:
-            result['edits'].to_sql('edits', con, if_exists='append', index=False)
-        if not result['commit'].empty:
-            result['commit'].to_sql('commits', con, if_exists='append', index=False)
+        with sqlite3.connect(sqlite_db_file) as con:
+            if not result['edits'].empty:
+                result['edits'].to_sql('edits', con, if_exists='append', index=False)
+            if not result['commit'].empty:
+                result['commit'].to_sql('commits', con, if_exists='append', index=False)
 
 
 def _process_repo_parallel(git_repo_dir, sqlite_db_file, commits, extraction_settings):
@@ -1164,16 +1163,16 @@ def _process_repo_parallel(git_repo_dir, sqlite_db_file, commits, extraction_set
         global git_init_lock
         git_init_lock = git_init_lock_
 
-    con = sqlite3.connect(sqlite_db_file)
     with multiprocessing.Pool(extraction_settings['no_of_processes'],
                               initializer=_init, initargs=(git_repo_dir,git_init_lock)) as p:
         with tqdm(total=len(args), desc='Parallel ({0} processes)' \
                   .format(extraction_settings['no_of_processes'])) as pbar:
             for result in p.imap_unordered(_process_commit, args, chunksize=extraction_settings['chunksize']):
-                if not result['edits'].empty:
-                    result['edits'].to_sql('edits', con, if_exists='append', index=False)
-                if not result['commit'].empty:
-                    result['commit'].to_sql('commits', con, if_exists='append', index=False)
+                with sqlite3.connect(sqlite_db_file) as con:
+                    if not result['edits'].empty:
+                        result['edits'].to_sql('edits', con, if_exists='append', index=False)
+                    if not result['commit'].empty:
+                        result['commit'].to_sql('commits', con, if_exists='append', index=False)
                 pbar.update(1)
 
 
@@ -1284,14 +1283,14 @@ def get_unified_changes(git_repo_dir, commit_hash, file_path):
 
     return df
 
-def check_mining_complete(git_repo_dir, sqlite_db_file, commits=[]):
+def check_mining_complete(git_repo_dir, sqlite_db_file, commits=[], all_branches=False, return_number_missing=False):
     """ Checks status of a mining operation
 
     Args:
         git_repo_dir: path to the git repository that is mined
         sqlite_db_file: path (including database name) where with sqlite database
         commits: only consider specific set of commits, considers all if empty
-        
+
     Returns:
         True if all commits are included in the database, otherwise False
     """
@@ -1310,14 +1309,20 @@ def check_mining_complete(git_repo_dir, sqlite_db_file, commits=[]):
         raise Exception("Found no database at provided path.")
 
     if not commits:
-        commits = [c.hash for c in git_repo.get_list_commits()]
+        commits = [c.hash for c in git_repo.get_list_commits(all=all_branches)]
     if set(commits).issubset(p_commits):
-        return True
+        if return_number_missing:
+            return (True, 0)
+        else:
+            return True
     else:
-        return False
-    
+        if return_number_missing:
+            return (False, len(set(commits).difference(p_commits)))
+        else:
+            return False
 
-def mining_state_summary(git_repo_dir, sqlite_db_file):
+
+def mining_state_summary(git_repo_dir, sqlite_db_file, all_branches=False):
     """ Prints mining progress of database and returns dataframe with details on missing commits.
 
     Args:
@@ -1341,7 +1346,7 @@ def mining_state_summary(git_repo_dir, sqlite_db_file):
     else:
         raise Exception("Found no database at provided path.")
 
-    commits = [c for c in git_repo.get_list_commits()]
+    commits = [c for c in git_repo.get_list_commits(all=all_branches)]
     if not p_commits.issubset({c.hash for c in commits}):
         raise Exception("The database does not match the provided repository.")
 
@@ -1397,7 +1402,8 @@ def mining_state_summary(git_repo_dir, sqlite_db_file):
 def mine_git_repo(git_repo_dir, sqlite_db_file, commits=[],
                   use_blocks=False, no_of_processes=os.cpu_count(), chunksize=1, exclude=[],
                   blame_C='', blame_w=False, max_modifications=0, timeout=0, extract_text=False,
-                  extract_complexity=False, extract_merges=True, extract_merge_deletions=False):
+                  extract_complexity=False, extract_merges=True, extract_merge_deletions=False,
+                  all_branches=False):
     """ Creates sqlite database with details on commits and edits for a given git repository.
 
     Args:
@@ -1420,16 +1426,20 @@ def mine_git_repo(git_repo_dir, sqlite_db_file, commits=[],
     Returns:
         sqlite database will be written at specified location
     """
-    git_version = check_output(['git', '--version']).strip().split()[-1].decode("utf-8")
+    git_version = check_output(['git', '--version']).strip().decode("utf-8")
 
-    if int(re.search(r'(\d+)(?:\.\d+[a-z]*)+', git_version).groups()[0]) < 2:
+    parsed_git_version = re.search(r'(\d+)\.(\d+)\.(\d+)', git_version).groups()
+
+    if int(parsed_git_version[0]) < 2 or ((int(parsed_git_version[0]) == 2) and \
+       ((int(parsed_git_version[0]) == 2) and (int(parsed_git_version[1]) < 11)) or \
+       (int(parsed_git_version[1]) == 11) and (int(parsed_git_version[2]) == 0)):
         raise Exception("Your system is using git " + git_version + " which is not supported by " +
-                        "git2net. Please update to git >= 2.0.")
+                        "git2net. Please update to git >= 2.11.1")
 
     blame_options = _parse_blame_C(blame_C) + ['--show-number', '--line-porcelain']
     if blame_w:
         blame_options += ['-w']
-        
+
     extraction_settings = {'use_blocks': use_blocks,
                            'no_of_processes': no_of_processes,
                            'chunksize': chunksize,
@@ -1443,6 +1453,7 @@ def mine_git_repo(git_repo_dir, sqlite_db_file, commits=[],
                            'extract_merge_deletions': extract_merge_deletions}
 
     git_repo = pydriller.GitRepository(git_repo_dir)
+
     if os.path.exists(sqlite_db_file):
         try:
             with sqlite3.connect(sqlite_db_file) as con:
@@ -1456,17 +1467,25 @@ def mine_git_repo(git_repo_dir, sqlite_db_file, commits=[],
                 if (prev_method == 'blocks' if use_blocks else 'lines') and \
                    (prev_extract_text == str(extract_text)):
                     try:
+                        # processed commits
                         p_commits = set(x[0]
                             for x in con.execute("SELECT hash FROM commits").fetchall())
                     except sqlite3.OperationalError:
                         p_commits = set()
-                    c_commits = set(c.hash
-                        for c in pydriller.GitRepository(git_repo_dir).get_list_commits())
+                    if all_branches:
+                        c_commits = set(c.hash for c in
+                                        pydriller.GitRepository(git_repo_dir).get_list_commits(all=True))
+                    else:
+                        c_commits = set(c.hash for c in
+                                        pydriller.GitRepository(git_repo_dir).get_list_commits())
                     if not p_commits.issubset(c_commits):
-                        raise Exception("Found a database that was created with identical " +
-                                        "settings. However, some commits in the database are not " +
-                                        "in the provided git repository. Please provide a clean " +
-                                        "database.")
+                        if prev_repository != git_repo.repo.remotes.origin.url:
+                            raise Exception("Found a database that was created with identical " +
+                                            "settings. However, some commits in the database are not " +
+                                            "in the provided git repository and the url of the current " +
+                                            "origin is different to the one listed in the database. " +
+                                            "Please provide a clean database or update the origin in " +
+                                            "the current database if you want to proceed.")
                     else:
                         if p_commits == c_commits:
                             print("The provided database is already complete!")
@@ -1504,30 +1523,50 @@ def mine_git_repo(git_repo_dir, sqlite_db_file, commits=[],
                                 :method,
                                 :extract_text)""",
                         {'version': 'git2net ' + str(__version__),
-                         'repository': git_repo_dir,
+                         'repository': git_repo.repo.remotes.origin.url,
                          'date': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                          'method': 'blocks' if use_blocks else 'lines',
                          'extract_text': str(extract_text)})
             con.commit()
-            p_commits = []
+            p_commits = set()
 
+
+    # commits in the currently mined repository
+    if all_branches:
+        c_commits = set(c.hash for c in
+                        pydriller.GitRepository(git_repo_dir).get_list_commits(all=True))
+    else:
+        c_commits = set(c.hash for c in
+                        pydriller.GitRepository(git_repo_dir).get_list_commits())
     if not commits:
-        u_commits = [c for c in git_repo.get_list_commits() if c.hash not in p_commits]
+        # unprocessed commits
+        if all_branches:
+            u_commits = [c for c in git_repo.get_list_commits(all=True) if c.hash not in p_commits]
+        else:
+            u_commits = [c for c in git_repo.get_list_commits() if c.hash not in p_commits]
     else:
-        c_commits = set(c.hash
-                        for c in pydriller.GitRepository(git_repo_dir).get_list_commits())
         if not set(commits).issubset(c_commits):
             raise Exception("At least one provided commit does not exist in the repository.")
         commits = [git_repo.get_commit(h) for h in commits]
         u_commits = [c for c in commits if c.hash not in p_commits]
 
+    current_branch = git_repo.repo.active_branch.name
+    for c in p_commits.intersection(c_commits):
+        b = con.execute("SELECT branches FROM commits WHERE hash = (:hash)", {'hash': c}).fetchall()[0][0]
+        b = set(b.split(','))
+        if current_branch not in b:
+            b.add(current_branch)
+            con.execute("UPDATE commits SET branches = (:branches) WHERE hash = (:hash)",
+                           {'branches': ','.join(b), 'hash': c})
+    con.commit()
+
     if extraction_settings['no_of_processes'] > 1:
         _process_repo_parallel(git_repo_dir, sqlite_db_file, u_commits, extraction_settings)
     else:
         _process_repo_serial(git_repo_dir, sqlite_db_file, u_commits, extraction_settings)
-        
-        
-        
+
+
+
 def mine_github(github_url, git_repo_dir, sqlite_db_file, branch=None, **kwargs):
     """ Clones a repository from github and starts the mining process.
 
@@ -1546,33 +1585,31 @@ def mine_github(github_url, git_repo_dir, sqlite_db_file, branch=None, **kwargs)
         sqlite database will be written at specified location
     """
     # github_url can either be provided as full url or as in form <USER>/<REPO>
-    user_repo_pattern = r'^([^\/\.]*)\/([^\/\.]*)$'
-    full_url_pattern = r'^https:\/\/github\.com\/([^\/\.]*)\/([^\/\.]*)(\.git)?$'
-    
+    user_repo_pattern = r'^([^\/]*)\/([^\/]*)$'
+    full_url_pattern = r'^https:\/\/github\.com\/([^\/]*)\/([^\/i]*)(\.git)?$'
+
     match = re.match(user_repo_pattern, github_url)
     if match:
-        print(match.groups())
         git_owner = match.groups()[0]
         git_repo = match.groups()[1]
         github_url = 'https://github.com/{}/{}'.format(git_owner, git_repo)
     else:
         match = re.match(full_url_pattern, github_url)
         if match:
-            print(match.groups())
             git_owner = match.groups()[0]
             git_repo = match.groups()[1]
         else:
-            raise Exception('Invalid github_url provided.')    
-    
+            raise Exception('Invalid github_url provided.')
+
     # detect the correct directories to work with
     local_directory = '/'.join(git_repo_dir.split('/')[:-1])
     git_repo_folder = git_repo_dir.split('/')[-1]
-    
+
     if local_directory == '':
         local_directory = '.'
     if git_repo_folder == '':
         git_repo_folder = git_owner + '__' + git_repo
-    
+
     # check if the folder is empty if it exists
     if os.path.exists(git_repo_dir) and \
        (len(os.listdir(os.path.join(local_directory, git_repo_folder))) > 0):
@@ -1582,6 +1619,6 @@ def mine_github(github_url, git_repo_dir, sqlite_db_file, branch=None, **kwargs)
             git.Git(local_directory).clone(github_url, git_repo_folder, branch=branch)
         else:
             git.Git(local_directory).clone(github_url, git_repo_folder)
-    
+
     # mine the cloned repo
     mine_git_repo(git_repo_dir, sqlite_db_file, **kwargs)
